#========================================================================================
# package      DE_EPAGES::AppStore::Hooks::Attribute
# state        private
#========================================================================================
package DE_EPAGES::AppStore::Hooks::Attribute;

use strict;
use DE_EPAGES::Object::API::Factory qw ( GetRootObjectID );
use DE_EPAGES::ThirdPartyConfig::API::Constants qw (ATTRIBUTES_CATEGORY_MISC);

#========================================================================================
# function     OnGetThirdPartyConfigAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnGetThirdPartyConfigAttributes($hParams);
#----------------------------------------------------------------------------------------
# description  API_GetThirdPartyConfigAttributes triggered, register attributes to be
#               configured in PBO
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>AttributesCategory
#                   <li>LanguageID
#               </ul> | ref.hash.*
#========================================================================================
sub OnGetThirdPartyConfigAttributes {
    my ($hParams) = @_;

    return unless $hParams->{'AttributesCategory'} eq ATTRIBUTES_CATEGORY_MISC;
    my $RootObjectID = GetRootObjectID();

    push @{$hParams->{'Attributes'}}, {
        'SectionID'   => 'AppStore',
        'SectionName' => 'AppStore',
        'Attributes'  => [
            { 'ObjectID' => $RootObjectID, 'AttributeAlias' => 'ShowAppStoreDisclaimer' },
            { 'ObjectID' => $RootObjectID, 'AttributeAlias' => 'ShowAppStorePriceOnAppDetails' }
        ]
    };

    return;
}

1;
