#========================================================================================
# package      DE_EPAGES::Colissimo::UI::ShippingMethodLevels
# state        private
#----------------------------------------------------------------------------------------
# description  User interface for ShippingMethodColissimo,
#
#========================================================================================
package DE_EPAGES::Colissimo::UI::ShippingMethodColissimo;
use base DE_EPAGES::ShippingTypes::UI::ShippingMethodLevels;

use strict;

use DE_EPAGES::Object::API::Table::Currency qw( GetInfoCurrency );

#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SaveSettings($Servlet);
#----------------------------------------------------------------------------------------
# description  Event handler
#----------------------------------------------------------------------------------------
# syntax       $Handler->SaveSettings($Servlet);
#------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;
    my $ShipMeth = $Servlet->object;

    # (1) save stuff
    $self->Save($Servlet);

    # (2) save "exemption" stuff (= Mindestbestellwert fr freie Lieferung)
    my $Shop     = $ShipMeth->getSite;
    my $TaxModel = $Shop->get('InputTaxModel');
    my $Form     = $Servlet->form->form($ShipMeth, 'SaveExemption');
    my %LevelsByCurrency;
    foreach my $hLevel (@{$Form->{'Exemption_ShippingLevels'}}) {
        my $currid = $hLevel->{'Exemption_CurrencyID'};
        my $upper  = $hLevel->{'Exemption_UpperBound'};
        my $levels = $LevelsByCurrency{ $currid } ||= [];
        push @$levels, { BaseValue => 0, UpperBound => $upper };
    }
    # save all levels in all currencies
    while ( my( $currid, $levels ) = each %LevelsByCurrency ) {
        $ShipMeth->setLimitLevel( $currid, $TaxModel, $levels, 1 );
    }

    return;
}

#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewSettings($Servlet);
#----------------------------------------------------------------------------------------
# description  Event handler
#------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub ViewSettings{
    my $self = shift;
    my ($Servlet) = @_;

    my $ShippingMethod = $Servlet->object;
    my $Shop = $Servlet->object->getSite();
    my $InputTaxModel = $Shop->get('InputTaxModel');

    my( $ColissimoCurrency, @Levels ); # values to determine
    my $aShopCurrencies = $Shop->get('Currencies');
    foreach (@$aShopCurrencies) {
        next if $_->{IsActivated} != 1;
        my $currid = $_->{CurrencyID};
        next if $currid ne 'EUR';
        $ColissimoCurrency = $_;
        # collect shipping levels, add empty rows for new currencies
        my $levels = $ShippingMethod->getLevels( $currid, $InputTaxModel, 1,1);
        my $hLevel;
        if ( @$levels ){
            my $tmp = $levels->[0]; # Why only the first one and not the rest?
            $hLevel = { map { 'Exemption_'.$_ => $tmp->{$_} } keys %$tmp };
        } else {
            $hLevel = {};
        }
        my $hCurrency = GetInfoCurrency( $currid );
        my %MyCurrency = map { 'Exemption_'.$_ => $hCurrency->{$_} } keys %$hCurrency;
        push @Levels, { %$hLevel, %MyCurrency };
    }

    $Servlet->vars('ColissimoCurrency', $ColissimoCurrency);
    $Servlet->vars('Exemption_ShippingLevels', \@Levels);

    return;
}

sub UnityViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    $self->ViewSettings($Servlet);
    $self->SUPER::UnityViewSettings($Servlet);
}


1;
