#========================================================================================
# package      DE_EPAGES::DHLIntraship::UI::ShippingLabeDHLIntraship
# base         DE_EPAGES::ExternalShipping::UI::ShippingLabel
# state        private
#========================================================================================
package DE_EPAGES::DHLIntraship::UI::ShippingLabelDHLIntraship;
use base DE_EPAGES::ExternalShipping::UI::ShippingLabel;
use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::Constants qw (
    %MAPPING_PRODUCTCODE_FULLPRODUCTNAME
    TOTAL_WEIGHT_TOO_HIGH_WARNING_KEYWORD
    TOTAL_WEIGHT_TOO_LOW_WARNING_KEYWORD
);
use DE_EPAGES::DHLIntraship::API::DHLIntrashipError qw ( FormErrorAndLogging );
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::CustomerAddress;
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products;
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel;
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShopAddress;

use feature 'switch';

use constant PACKSTATION_NAME => 'Packstation';


sub Save {
    my $self           = shift;
    my ($Servlet)      = @_;

    my $ShippingLabel = $Servlet->object();
    my $ProductCodeID = $ShippingLabel->get('ShippingFulfillmentMethod')->get('TeilnehmerNational');
    my $ProductCode = $MAPPING_PRODUCTCODE_FULLPRODUCTNAME{$ProductCodeID};

    $ShippingLabel->set({'ProductCode'=> $ProductCode});

    my $hServletParams = $Servlet->params();
    $self->_updatePackstationInfo($hServletParams, $ShippingLabel);

    $self->SUPER::Save($Servlet);

    # Validation to provide warnings (don't prevent from saving data)
    eval {
        $Servlet = $self->_Validation($Servlet);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        FormErrorAndLogging('ValidationError',$Servlet,$Error);
    }
    $Servlet->form->executeFormError;

    return;
}

#----------------------------------------------------------------------------------------
# Private methods
#----------------------------------------------------------------------------------------
sub _updateRecipient {
    my $self = shift;
    my ($ShippingAddress, $IsPickupAddress) = @_;

    my $CurrentIsPickupAddress = $ShippingAddress->get('IsPickupAddress');

    return if ($CurrentIsPickupAddress eq $IsPickupAddress) || $IsPickupAddress;

    if (uc($ShippingAddress->get('FirstName')) eq uc(PACKSTATION_NAME)) {
        $ShippingAddress->set({
            'FirstName' => undef,
            'LastName' => undef,
        });
    }

    return;
}

sub _updatePackstationInfo {
    my $self = shift;
    my ($hServletParams, $ShippingLabel) = @_;

    my $IsPickupAddress = (defined($hServletParams->{'IsPickupAddress'}) && $hServletParams->{'IsPickupAddress'} eq 'on') ? 1 : 0;
    my $DHLPostNumber = $hServletParams->{'DHLPostNumber'};
    my $ShippingAddress = $ShippingLabel->get('Order')->get('ShippingAddress');

    return if !isValid($ShippingAddress);

    $self->_updateRecipient($ShippingAddress, $IsPickupAddress);

    if (isValid($ShippingAddress)) {
        $ShippingAddress->set({'IsPickupAddress' => $IsPickupAddress});
        if (isValid($DHLPostNumber)) {
            $ShippingAddress->set({'DHLPostNumber' => $DHLPostNumber});
        }
    }

    return;
}

sub _Validation {
    my $self = shift;
    my ($Servlet) = @_;
    my $ShippingLabel = $Servlet->object();
    my $ShopAddress = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShopAddress->new( $ShippingLabel );
    if ( $ShopAddress->hasWarningStates() ){
        foreach my $Warning (@{$ShopAddress->getStatesAsWarning()}){
            $Servlet->form->mapFormError(
                {
                    'Name'=>'ShopAddress',
                    'WarningAttribute' => $Warning->{'Vars'}->{'attribute'},
                    'WarningReason' => $Warning->{'Vars'}->{'reason'}
                }, $Warning
            );
        }
    }
    my $CustomerAddress = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::CustomerAddress->new( $ShippingLabel );
    if ( $CustomerAddress->hasWarningStates() ){
        foreach my $Warning (@{$CustomerAddress->getStatesAsWarning()}){
            $Servlet->form->mapFormError(
                {
                    'Name'=>'CustomerAddress',
                    'WarningAttribute' => $Warning->{'Vars'}->{'attribute'},
                    'WarningReason' => $Warning->{'Vars'}->{'reason'}
                }, $Warning);
        }
    }
    my $ShippingLabelValidaton = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $ShippingLabel );
    unless ( $ShippingLabelValidaton->isComplete() ){
        foreach my $Warning (@{$ShippingLabelValidaton->getStatesAsWarning()}){
            $Servlet->form->mapFormError(
                {
                    'Name'=>'ShippingLabel',
                    'WarningAttribute' => $Warning->{'Vars'}->{'attribute'},
                    'WarningReason' => $Warning->{'Vars'}->{'reason'}
                }, $Warning);
        }
    }
    my $Products = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products->new( $ShippingLabel );
    unless ( $Products->isComplete() ){
        foreach my $Warning (@{$Products->getStatesAsWarning()}){
            if ( ($Warning->{'Vars'}->{'reason'} eq TOTAL_WEIGHT_TOO_HIGH_WARNING_KEYWORD)
              || ($Warning->{'Vars'}->{'reason'} eq TOTAL_WEIGHT_TOO_LOW_WARNING_KEYWORD) ){
                $Servlet->form->mapFormError(
                    {
                        'Name'=>'TotalWeight',
                        'WarningAttribute' => $Warning->{'Vars'}->{'attribute'},
                        'WarningReason' => $Warning->{'Vars'}->{'reason'}
                    }, $Warning);
            } else {
                $Servlet->form->mapFormError(
                    {
                        'Name'=>'ShippingLabel',
                        'WarningAttribute' => $Warning->{'Vars'}->{'attribute'},
                        'WarningReason' => $Warning->{'Vars'}->{'reason'}
                    }, $Warning);
            }
        }
    }

    return $Servlet;
}

1;