#========================================================================================
# package      DE_EPAGES::Mollie::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Mollie::API::Cartridge;
use base DE_EPAGES::Payment::API::PaymentInstaller;

use strict;



#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Mollie::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Mollie.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Mollie',
        'Version'            => '7.26.0',
        'Patches'            => ['6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0'],
    );

    return bless $self, $class;
}



#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall();
    $self->SUPER::install();
    $self->finishInstall();

    return;
}



#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall();
    $self->SUPER::uninstall();
    $self->finishUninstall();

    return;
}

1;