#========================================================================================
# package      DE_EPAGES::ProductPortal::t::unit::PortalConfig
# base         DE_EPAGES::base
#----------------------------------------------------------------------------------------
# description  unit test for API::Object::PortalConfig
#========================================================================================
package DE_EPAGES::ProductPortal::t::unit::PortalConfig;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_isConfigured_no_products_to_export();
    $self->test_isConfigured_all_products_to_export();
    $self->test_isConfigured_some_products_to_export();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_isConfigured_no_products_to_export {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $ExportAllProducts = 0;
    my $CountProducts = 0;
    my $PortalConfig = $self->_createPortalConfig($ExportAllProducts, $CountProducts);
    ok(!$PortalConfig->isConfigured(), "$SubTestName - Portal config IS NOT configured when no products selected to export.");
}

#----------------------------------------------------------------------------------------
sub test_isConfigured_all_products_to_export {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $ExportAllProducts = 1;
    my $CountProducts = 0;
    my $PortalConfig = $self->_createPortalConfig($ExportAllProducts, $CountProducts);
    ok($PortalConfig->isConfigured(), "$SubTestName - Portal config IS configured when all products option is marked to export.");
}

#----------------------------------------------------------------------------------------
sub test_isConfigured_some_products_to_export {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $ExportAllProducts = 0;
    my $CountProducts = 5;
    my $PortalConfig = $self->_createPortalConfig($ExportAllProducts, $CountProducts);
    ok($PortalConfig->isConfigured(), "$SubTestName - Portal config IS configured when some products are added to export.");
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createPortalConfig {
    my $self = shift;
    my ($ExportAllProducts, $CountProducts) = @_;

    my $hParameterList = {
        'CountProducts' => $CountProducts,
        'ExportAllProducts' => $ExportAllProducts
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::ProductPortal::API::Object::PortalConfig',
            $hParameterList
    );

    return $Mockify->getMockObject();
}


__PACKAGE__->runScript();
