#========================================================================================
# package      DE_EPAGES::ProductPortal::t::unit::UI_ProductPortal
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for UI::ProductPortal
#========================================================================================
package DE_EPAGES::ProductPortal::t::unit::UI_ProductPortal;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Core::API::PerlTools qw (GetCurrentDateTime SubtractDuration);
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetCallCount );
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Mockify::Matcher qw ( HashRef Object String );
use Test::Mockify::Sut;
use Test::More;

use strict;

use constant PORTAL_SITE_ID => '1222';
use constant PORTAL_CONFIG_ID => '1234';

sub test {
    my $self = shift;

    $self->test_UpdatePortalConfig_add_PortalConfig();
    $self->test_UpdatePortalConfig_add_PortalConfig_deprecated();
    $self->test_UpdatePortalConfig_add_and_delete_PortalConfigs();
    $self->test_UpdatePortalConfig_add_and_delete_PortalConfigs_deprecated();
    $self->test_UpdatePortalConfig_delete_PortalConfig();
    $self->test_UpdatePortalConfig_delete_PortalConfig_deprecated();
    $self->test_GeneratePortalConfigProductPrettyUnityUrls_no_unity_shop();
    $self->test_GeneratePortalConfigProductPrettyUnityUrls_is_unity_shop_no_portalconfig_completed();
    $self->test_GeneratePortalConfigProductPrettyUnityUrls_is_unity_shop_is_portalconfig_completed();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_add_PortalConfig {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AddPortalConfig = 1;
    my $DeletePortalConfig = 0;
    my $IsPortalSiteDeprecated = 0;

    my $Servlet = $self->_createServletOnShop($IsPortalSiteDeprecated, $AddPortalConfig, $DeletePortalConfig);
    my $SUT = $self->_createSUT();

    $SUT->UpdatePortalConfig($Servlet);

    is($Servlet->vars('DELETE'), undef, "$SubTestName - There are NOT portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 1, "$SubTestName - Portal config creation method DOES gets called.");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_add_PortalConfig_deprecated {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AddPortalConfig = 1;
    my $DeletePortalConfig = 0;
    my $IsPortalSiteDeprecated = 1;

    my $Servlet = $self->_createServletOnShop($IsPortalSiteDeprecated, $AddPortalConfig, $DeletePortalConfig);
    my $SUT = $self->_createSUT();

    $SUT->UpdatePortalConfig($Servlet);

    is($Servlet->vars('DELETE'), undef, "$SubTestName - There are NOT portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 0, "$SubTestName - Portal config creation method does NOT gets called.");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_add_and_delete_PortalConfigs {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AddPortalConfig = 1;
    my $DeletePortalConfig = 1;
    my $IsPortalSiteDeprecated = 0;

    my $Servlet = $self->_createServletOnShop($IsPortalSiteDeprecated, $AddPortalConfig, $DeletePortalConfig);
    my $SUT = $self->_createSUT();

    $SUT->UpdatePortalConfig($Servlet);

    is($Servlet->vars('DELETE'), 1, "$SubTestName - There ARE portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 1, "$SubTestName - Portal config creation method DOES gets called.");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_add_and_delete_PortalConfigs_deprecated {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AddPortalConfig = 1;
    my $DeletePortalConfig = 1;
    my $IsPortalSiteDeprecated = 1;

    my $Servlet = $self->_createServletOnShop($IsPortalSiteDeprecated, $AddPortalConfig, $DeletePortalConfig);
    my $SUT = $self->_createSUT();

    $SUT->UpdatePortalConfig($Servlet);

    is($Servlet->vars('DELETE'), 1, "$SubTestName - There ARE portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 0, "$SubTestName - Portal config creation method does NOT gets called (deprecated).");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_delete_PortalConfig {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AddPortalConfig = 0;
    my $DeletePortalConfig = 1;
    my $IsPortalSiteDeprecated = 0;

    my $Servlet = $self->_createServletOnShop($IsPortalSiteDeprecated, $AddPortalConfig, $DeletePortalConfig);
    my $SUT = $self->_createSUT();

    $SUT->UpdatePortalConfig($Servlet);

    is($Servlet->vars('DELETE'), 1, "$SubTestName - There ARE portal configs to delete.");
    is(GetCallCount($SUT, '_createPortalConfig'), 0, "$SubTestName - Portal config creation method does NOT gets called.");
}

#----------------------------------------------------------------------------------------
sub test_UpdatePortalConfig_delete_PortalConfig_deprecated {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AddPortalConfig = 0;
    my $DeletePortalConfig = 1;
    my $IsPortalSiteDeprecated = 1;

    my $Servlet = $self->_createServletOnShop($IsPortalSiteDeprecated, $AddPortalConfig, $DeletePortalConfig);
    my $SUT = $self->_createSUT();

    $SUT->UpdatePortalConfig($Servlet);

    is($Servlet->vars('DELETE'), 1, "$SubTestName - There ARE portal configs to delete (deprecation does not prevents deletion).");
}

#----------------------------------------------------------------------------------------
sub test_GeneratePortalConfigProductPrettyUnityUrls_no_unity_shop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ExportAllProducts = 1;
    my $IsUnityShop = 0;

    my $Servlet = $self->_createServletOnPortalConfig($ExportAllProducts, $IsUnityShop);
    my $SUT = $self->_createSUT();

    $SUT->GeneratePortalConfigProductPrettyUnityUrls($Servlet);

    is(GetCallCount($SUT, '_GetAndSaveProductPrettyUnityUrls'), 0, "$SubTestName - Portal config pretty url request generation does NOT gets called.");
}

#----------------------------------------------------------------------------------------
sub test_GeneratePortalConfigProductPrettyUnityUrls_is_unity_shop_no_portalconfig_completed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ExportAllProducts = 0;
    my $IsUnityShop = 1;

    my $Servlet = $self->_createServletOnPortalConfig($ExportAllProducts, $IsUnityShop);
    my $SUT = $self->_createSUT();

    $SUT->GeneratePortalConfigProductPrettyUnityUrls($Servlet);

    is(GetCallCount($SUT, '_GetAndSaveProductPrettyUnityUrls'), 0, "$SubTestName - Portal config pretty url request generation does NOT gets called.");
}

#----------------------------------------------------------------------------------------
sub test_GeneratePortalConfigProductPrettyUnityUrls_is_unity_shop_is_portalconfig_completed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ExportAllProducts = 1;
    my $IsUnityShop = 1;

    my $Servlet = $self->_createServletOnPortalConfig($ExportAllProducts, $IsUnityShop);
    my $SUT = $self->_createSUT();

    $SUT->GeneratePortalConfigProductPrettyUnityUrls($Servlet);

    is(GetCallCount($SUT, '_GetAndSaveProductPrettyUnityUrls'), 1, "$SubTestName - Portal config pretty url request generation DOES gets called.");
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = Test::Mockify::Sut->new( 'DE_EPAGES::ProductPortal::UI::PortalConfig', $aParameterList );

    $Mockify->spy('_createPortalConfig')
        ->when(
            Object('DE_EPAGES::Presentation::Servlet::Permission'),
            Object('DE_EPAGES::ProductPortal::API::Object::PortalSite'),
            Object('DE_EPAGES::Object::API::Object::Object'),
            Object('DE_EPAGES::Object::API::Object::Class'));

    $Mockify->mock('NewObject')
        ->when(
            Object('DE_EPAGES::Presentation::Servlet::Permission'),
            Object('DE_EPAGES::Object::API::Object::Object'),
            Object('DE_EPAGES::Object::API::Object::Class'),
            HashRef()
        )->thenReturnUndef();

    $Mockify->spy('_GetAndSaveProductPrettyUnityUrls')
        ->when(
            Object('DE_EPAGES::Shop::API::Object::Shop'),
            Object('DE_EPAGES::ProductPortal::API::Object::PortalConfig'));

    $Mockify->mockImported('DE_EPAGES::Product::API::Product', 'GetProductPrettyUnityUrlsRequest')
        ->whenAny()
        ->thenReturn([]);

    $Mockify->mockImported('DE_EPAGES::Product::API::Product', 'SaveProductPrettyUrlsResponse')
        ->whenAny()
        ->thenReturn({});

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createServletOnShop {
    my $self = shift;
    my ($IsPortalSiteDeprecated, $AddPortalConfig, $DeletePortalConfig) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyServlet->new( $self->_createShop_ShopServlet($IsPortalSiteDeprecated, $DeletePortalConfig) );
    $Mockify->mock('form',$self->_createForm($AddPortalConfig, $DeletePortalConfig));
    $Mockify->mock('user',$self->_createFakeUserObject());
    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShop_ShopServlet {
    my $self = shift;
    my ($IsPortalSiteDeprecated, $DeletePortalConfig) = @_;

    my $aPortalConfigs = ($DeletePortalConfig) ? [ $self->_createPortalConfig_ShopServlet(PORTAL_CONFIG_ID) ] : [];
    my $hAttributes = {
        'Alias' => 'MyShop',
        'IsUnityRegistered' => 0,
        'PortalConfigs' => $aPortalConfigs
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop',$hAttributes);
    my $aParameterTypes = [ 'int', {'string' => 'Object'}, 'hashref' ];
    my $PortalSite = $self->_createPortalSite($IsPortalSiteDeprecated);
    $Mockify->addMockWithReturnValueAndParameterCheck('loadObject', $PortalSite, $aParameterTypes);

    $aParameterTypes = [ {'string' => 'PortalConfigs'}, 'hashref' ];
    $Mockify->addMockWithReturnValueAndParameterCheck('folder', $self->_createFolderObject(), $aParameterTypes);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPortalConfig_ShopServlet {
    my $self = shift;
    my ($PortalConfigId) = @_;

    my $hAttributes = {
        'ID' => $PortalConfigId
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::PortalConfig',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;
    my ($AddPortalConfig, $DeletePortalConfig) = @_;

    my $aParameterList = [];
    my $Mockify = Test::Mockify->new(
                   'DE_EPAGES::Presentation::API::Form',
                   $aParameterList
              );
    my @ListOfPortalConfigsAndSites = ($DeletePortalConfig) ? () : (PORTAL_CONFIG_ID); # unchecked if going to be deleted
    push(@ListOfPortalConfigsAndSites, PORTAL_SITE_ID) if ($AddPortalConfig); # checked if going to be added
    if (scalar(@ListOfPortalConfigsAndSites) > 0) {
        $Mockify
            ->mock('value')
            ->when( String('PortalID') )->thenReturn(@ListOfPortalConfigsAndSites);
    } else {
        $Mockify
            ->mock('value')
            ->when( String('PortalID') )->thenReturnUndef();
    }
    $Mockify
        ->mock('value')
        ->when( String('ACCEPT') )->thenReturnUndef();
    $Mockify
        ->mock('value')
        ->when( String('OnlyAmazon') )->thenReturn(0);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPortalSite {
    my $self = shift;
    my ($IsPortalSiteDeprecated) = @_;

    my $DateInThePast = SubtractDuration(GetCurrentDateTime(), 'months', 1);
    my $DeprecationDate = $IsPortalSiteDeprecated ? $DateInThePast : undef;
    my $hAttributes = {
        'Alias' => 'MyPortalSite',
        'DeprecationDate' => $DeprecationDate,
        'ID' => PORTAL_SITE_ID,
        'instanceOf' => 'PortalSite',
        'LocaleID' => 'de_DE',
        'Portal' => $self->_createPortal,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::PortalSite',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPortal {
    my $self = shift;

    my $hAttributes = {
        'PortalConfigClass' => $self->_createPortalConfigClass()
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::PortalSite',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createFolderObject {
    my $self = shift;

    my $hParameters = {
        'Child' => {}
    };
    my $EpagesMockObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Object', $hParameters);

    return $EpagesMockObject;
}

#----------------------------------------------------------------------------------------
sub _createPortalConfigClass {
    my $self = shift;

    my $hAttributes = {};
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', $hAttributes);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createFakeUserObject {
    my $self = shift;

    my $hAttributes = {
        'DeleteConfirmation' => 1
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object', $hAttributes);

    return $Mockify->getMockObject();
}


#----------------------------------------------------------------------------------------
sub _createServletOnPortalConfig {
    my $self = shift;
    my ($ExportAllProducts, $IsUnityShop) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyServlet->new(
            $self->_createPortalConfig_PortalConfigServlet($ExportAllProducts, $IsUnityShop)
    );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPortalConfig_PortalConfigServlet {
    my $self = shift;
    my ($ExportAllProducts, $IsUnityShop) = @_;

    my $hAttributes = {
        'CountProducts' => 0,
        'ExportAllProducts' => $ExportAllProducts,
        'LanguageID' => 1,
        'PortalSite' => $self->_createPortalSite(),
        'Site' => $self->_createShop_PortalConfigServlet($IsUnityShop)
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::PortalConfig',
        $hAttributes
    );
    $Mockify->addMockWithReturnValue('items', []);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShop_PortalConfigServlet {
    my $self = shift;
    my ($IsUnityShop) = @_;

    my $hAttributes = {
        'IsUnityRegistered' => $IsUnityShop,
        'LanguageID' => 1,
        'LocaleID' => 'de_DE'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop',$hAttributes);
    my $aParameterTypes = [ {'string' => 'POST'}, {'string' => '/products/get-urls'}, 'hashref', 'arrayref', 'arrayref' ];
    $Mockify->addMockWithReturnValueAndParameterCheck('sendUnityRequest', [], $aParameterTypes);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();