#========================================================================================
# package      DE_EPAGES::WebInterface::t::MessageCenterCacheUpdate
# state        private
#========================================================================================
package DE_EPAGES::WebInterface::t::MessageCenterCacheUpdate;
use base DE_EPAGES::Core::API::Test;
use threads;
use strict;
use warnings;
use feature qw(switch say);

use Test::More;
use DE_EPAGES::DataCache::API::Cache qw( GetCache GetTouched UpdateCache );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::WebInterface::API::ServerConfig qw( ServerConfig );

my $Debug = 0;

__PACKAGE__->runScript;

sub test {
    my $Mode = $ARGV[0];
    given( $Mode ) {
        when( undef ) {
            # test mode

            my $AppServerThread = async {
                eval {
                    AppServer();
                };
                print STDERR "Error in AppServer thread: $@" if $@;
            };

            sleep 1; # wait for the server tor start
            Script(); # run client test cases
            $AppServerThread->join; # wait for the server to shutdown
            ok( 1, 'server has finished' );
        }
        when( 'server' ) {
            AppServer();
        }
        when( 'script' ) {
            Script();
        }
    }
    done_testing();
}

sub AppServer {

    is_deeply( GetTouched(), {}, 'touched cache is initially empty' );
    is_deeply( GetCache('TestStore', 'InfoProduct'), {}, 'local cache is initially empty' );

    say 'Connect to MC' if $Debug;
    my $MessageCenter = DE_EPAGES::WebInterface::API::MessageCenter->new(
        AppServerAddress => '127.0.0.1',
        AppServerPort => '10048',
        PoolName => ( keys %{ ServerConfig()->pools } )[0],
    );
    $MessageCenter->connect;
    $MessageCenter->sendInitialFreeRequest;

    UpdateCache('TestStore', 'InfoProduct', '0815');
    is( GetTouched()->{All}{TestStore}{InfoProduct}{'0815'}, 1, '1 row changed' );

    GetCache('TestStore', 'InfoProduct')->{'0815'} = { ProductID => '0815', Weight => 25.4 };
    is( GetCache('TestStore', 'InfoProduct')->{'0815'}->{ProductID}, '0815', '1 row in local cache' );

    sleep 1; # wait for script to modify something

    # overload the UpdateTouched() function, which is called by sendIdleRequest()
    my $UpdateTouched = *DE_EPAGES::DataCache::API::Cache::UpdateTouched;
    local *DE_EPAGES::WebInterface::API::MessageCenter::UpdateTouched = sub {
        my ($hModified) = @_;
        is( $hModified->{All}{TestStore}{InfoProduct}{'Gold'}, 1, 'product "Gold" was modified by script' );
        $UpdateTouched->( $hModified );
    };

    $MessageCenter->sendIdleRequest;
    GetCache('TestStore', 'InfoProduct')->{'0815'} = { ProductID => '0815', Weight => 25.4 };

    is_deeply( GetTouched(), {}, 'touched cache is empty again' );

#    sleep 10;

    $MessageCenter->sendFinishRequest;
    ok (1,'finish');

    say 'Server shut down' if $Debug;;
}

sub Script {

    is_deeply( GetTouched(), {}, 'touched cache is initially empty' );

    UpdateCache('TestStore', 'InfoProduct', 'Gold');
    is( GetTouched()->{All}{TestStore}{InfoProduct}{'Gold'}, 1, 'product "Gold" changed' );

    SynchronizeCache();
    is_deeply( GetTouched(), {}, 'touched cache is empty after SynchronizeCache' );
}
